@@ -2,8 +2,8 @@
  *
  * BROKER.C - Event broker routines for Nagios
  *
- * Copyright (c) 2002-2005 Ethan Galstad (nagios@nagios.org)
- * Last Modified:   12-16-2005
+ * Copyright (c) 2002-2006 Ethan Galstad (nagios@nagios.org)
+ * Last Modified:   10-18-2007
  *
  * License:
  *
@@ -105,6 +105,7 @@ void broker_timed_event(int type, int flags, int attr, timed_event *event, struc
 	ds.recurring=event->recurring;
 	ds.run_time=event->run_time;
 	ds.event_data=event->event_data;
+	ds.event_ptr=(void *)event;
 
 	/* make callbacks */
 	neb_make_callbacks(NEBCALLBACK_TIMED_EVENT_DATA,(void *)&ds);
@@ -173,7 +174,7 @@ void broker_system_command(int type, int flags, int attr, struct timeval start_t
 
 
 /* send event handler data to broker */
-void broker_event_handler(int type, int flags, int attr, int eventhandler_type, void *data, int state, int state_type, struct timeval start_time, struct timeval end_time, double exectime, int timeout, int early_timeout, int retcode, char *command, char *cmdline, char *output, struct timeval *timestamp){
+void broker_event_handler(int type, int flags, int attr, int eventhandler_type, void *data, int state, int state_type, struct timeval start_time, struct timeval end_time, double exectime, int timeout, int early_timeout, int retcode, char *cmd, char *cmdline, char *output, struct timeval *timestamp){
 	service *temp_service=NULL;
 	host *temp_host=NULL;
 	char *command_buf=NULL;
@@ -188,8 +189,8 @@ void broker_event_handler(int type, int flags, int attr, int eventhandler_type,
 		return;
 
 	/* get command name/args */
-	if(command!=NULL){
-		command_buf=strdup(command);
+	if(cmd!=NULL){
+		command_buf=(char *)strdup(cmd);
 		command_name=strtok(command_buf,"!");
 		command_args=strtok(NULL,"\x0");
 	        }
@@ -211,6 +212,7 @@ void broker_event_handler(int type, int flags, int attr, int eventhandler_type,
 		ds.host_name=temp_host->name;
 		ds.service_description=NULL;
 	        }
+	ds.object_ptr=data;
 	ds.state=state;
 	ds.state_type=state_type;
 	ds.start_time=start_time;
@@ -228,7 +230,7 @@ void broker_event_handler(int type, int flags, int attr, int eventhandler_type,
 	neb_make_callbacks(NEBCALLBACK_EVENT_HANDLER_DATA,(void *)&ds);
 
 	/* free memory */
-	free(command_buf);
+	my_free(command_buf);
 
 	return;
         }
@@ -237,21 +239,22 @@ void broker_event_handler(int type, int flags, int attr, int eventhandler_type,
 
 
 /* send host check data to broker */
-void broker_host_check(int type, int flags, int attr, host *hst, int check_type, int state, int state_type, struct timeval start_time, struct timeval end_time, char *command, double latency, double exectime, int timeout, int early_timeout, int retcode, char *cmdline, char *output, char *perfdata, struct timeval *timestamp){
+int broker_host_check(int type, int flags, int attr, host *hst, int check_type, int state, int state_type, struct timeval start_time, struct timeval end_time, char *cmd, double latency, double exectime, int timeout, int early_timeout, int retcode, char *cmdline, char *output, char *long_output, char *perfdata, struct timeval *timestamp){
 	char *command_buf=NULL;
 	char *command_name=NULL;
 	char *command_args=NULL;
 	nebstruct_host_check_data ds;
+	int return_code=OK;
 
 	if(!(event_broker_options & BROKER_HOST_CHECKS))
-		return;
+		return OK;
 	
 	if(hst==NULL)
-		return;
+		return ERROR;
 
 	/* get command name/args */
-	if(command!=NULL){
-		command_buf=strdup(command);
+	if(cmd!=NULL){
+		command_buf=(char *)strdup(cmd);
 		command_name=strtok(command_buf,"!");
 		command_args=strtok(NULL,"\x0");
 	        }
@@ -263,6 +266,7 @@ void broker_host_check(int type, int flags, int attr, host *hst, int check_type,
 	ds.timestamp=get_broker_timestamp(timestamp);
 
 	ds.host_name=hst->name;
+	ds.object_ptr=(void *)hst;
 	ds.check_type=check_type;
 	ds.current_attempt=hst->current_attempt;
 	ds.max_attempts=hst->max_attempts;
@@ -279,35 +283,37 @@ void broker_host_check(int type, int flags, int attr, host *hst, int check_type,
 	ds.latency=latency;
 	ds.return_code=retcode;
 	ds.output=output;
+	ds.long_output=long_output;
 	ds.perf_data=perfdata;
 
 	/* make callbacks */
-	neb_make_callbacks(NEBCALLBACK_HOST_CHECK_DATA,(void *)&ds);
+	return_code=neb_make_callbacks(NEBCALLBACK_HOST_CHECK_DATA,(void *)&ds);
 
 	/* free data */
-	free(command_buf);
+	my_free(command_buf);
 
-	return;
+	return return_code;
         }
 
 
 
 /* send service check data to broker */
-void broker_service_check(int type, int flags, int attr, service *svc, int check_type, struct timeval start_time, struct timeval end_time, char *command, double latency, double exectime, int timeout, int early_timeout, int retcode, char *cmdline, struct timeval *timestamp){
+int broker_service_check(int type, int flags, int attr, service *svc, int check_type, struct timeval start_time, struct timeval end_time, char *cmd, double latency, double exectime, int timeout, int early_timeout, int retcode, char *cmdline, struct timeval *timestamp){
 	char *command_buf=NULL;
 	char *command_name=NULL;
 	char *command_args=NULL;
 	nebstruct_service_check_data ds;
+	int return_code=OK;
 
 	if(!(event_broker_options & BROKER_SERVICE_CHECKS))
-		return;
+		return OK;
 	
 	if(svc==NULL)
-		return;
+		return ERROR;
 
 	/* get command name/args */
-	if(command!=NULL){
-		command_buf=strdup(command);
+	if(cmd!=NULL){
+		command_buf=(char *)strdup(cmd);
 		command_name=strtok(command_buf,"!");
 		command_args=strtok(NULL,"\x0");
 	        }
@@ -320,6 +326,7 @@ void broker_service_check(int type, int flags, int attr, service *svc, int check
 
 	ds.host_name=svc->host_name;
 	ds.service_description=svc->description;
+	ds.object_ptr=(void *)svc;
 	ds.check_type=check_type;
 	ds.current_attempt=svc->current_attempt;
 	ds.max_attempts=svc->max_attempts;
@@ -336,15 +343,16 @@ void broker_service_check(int type, int flags, int attr, service *svc, int check
 	ds.latency=latency;
 	ds.return_code=retcode;
 	ds.output=svc->plugin_output;
+	ds.long_output=svc->long_plugin_output;
 	ds.perf_data=svc->perf_data;
 
 	/* make callbacks */
-	neb_make_callbacks(NEBCALLBACK_SERVICE_CHECK_DATA,(void *)&ds);
+	return_code=neb_make_callbacks(NEBCALLBACK_SERVICE_CHECK_DATA,(void *)&ds);
 
 	/* free data */
-	free(command_buf);
+	my_free(command_buf);
 
-	return;
+	return return_code;
         }
 
 
@@ -366,6 +374,7 @@ void broker_comment_data(int type, int flags, int attr, int comment_type, int en
 	ds.entry_type=entry_type;
 	ds.host_name=host_name;
 	ds.service_description=svc_description;
+	ds.object_ptr=NULL; /* not implemented yet */
 	ds.entry_time=entry_time;
 	ds.author_name=author_name;
 	ds.comment_data=comment_data;
@@ -399,6 +408,7 @@ void broker_downtime_data(int type, int flags, int attr, int downtime_type, char
 	ds.downtime_type=downtime_type;
 	ds.host_name=host_name;
 	ds.service_description=svc_description;
+	ds.object_ptr=NULL; /* not implemented yet */
 	ds.entry_time=entry_time;
 	ds.author_name=author_name;
 	ds.comment_data=comment_data;
@@ -448,6 +458,7 @@ void broker_flapping_data(int type, int flags, int attr, int flapping_type, void
 		ds.service_description=NULL;
 		ds.comment_id=temp_host->flapping_comment_id;
 	        }
+	ds.object_ptr=data;
 	ds.percent_change=percent_change;
 	ds.high_threshold=high_threshold;
 	ds.low_threshold=low_threshold;
@@ -547,6 +558,29 @@ void broker_service_status(int type, int flags, int attr, service *svc, struct t
 
 
 
+/* sends contact status updates to broker */
+void broker_contact_status(int type, int flags, int attr, contact *cntct, struct timeval *timestamp){
+	nebstruct_service_status_data ds;
+
+	if(!(event_broker_options & BROKER_STATUS_DATA))
+		return;
+
+	/* fill struct with relevant data */
+	ds.type=type;
+	ds.flags=flags;
+	ds.attr=attr;
+	ds.timestamp=get_broker_timestamp(timestamp);
+
+	ds.object_ptr=(void *)cntct;
+
+	/* make callbacks */
+	neb_make_callbacks(NEBCALLBACK_CONTACT_STATUS_DATA,(void *)&ds);
+
+	return;
+        }
+
+
+
 /* send notification data to broker */
 void broker_notification_data(int type, int flags, int attr, int notification_type, int reason_type, struct timeval start_time, struct timeval end_time, void *data, char *ack_author, char *ack_data, int escalated, int contacts_notified, struct timeval *timestamp){
 	nebstruct_notification_data ds;
@@ -580,6 +614,7 @@ void broker_notification_data(int type, int flags, int attr, int notification_ty
 		ds.state=temp_host->current_state;
 		ds.output=temp_host->plugin_output;
 	        }
+	ds.object_ptr=data;
 	ds.ack_author=ack_author;
 	ds.ack_data=ack_data;
 	ds.escalated=escalated;
@@ -627,6 +662,8 @@ void broker_contact_notification_data(int type, int flags, int attr, int notific
 		ds.state=temp_host->current_state;
 		ds.output=temp_host->plugin_output;
 	        }
+	ds.object_ptr=data;
+	ds.contact_ptr=(void *)cntct;
 	ds.ack_author=ack_author;
 	ds.ack_data=ack_data;
 	ds.escalated=escalated;
@@ -639,7 +676,7 @@ void broker_contact_notification_data(int type, int flags, int attr, int notific
 
 
 /* send contact notification data to broker */
-void broker_contact_notification_method_data(int type, int flags, int attr, int notification_type, int reason_type, struct timeval start_time, struct timeval end_time, void *data, contact *cntct, char *command, char *ack_author, char *ack_data, int escalated, struct timeval *timestamp){
+void broker_contact_notification_method_data(int type, int flags, int attr, int notification_type, int reason_type, struct timeval start_time, struct timeval end_time, void *data, contact *cntct, char *cmd, char *ack_author, char *ack_data, int escalated, struct timeval *timestamp){
 	nebstruct_contact_notification_method_data ds;
 	host *temp_host=NULL;
 	service *temp_service=NULL;
@@ -651,8 +688,8 @@ void broker_contact_notification_method_data(int type, int flags, int attr, int
 		return;
 
 	/* get command name/args */
-	if(command!=NULL){
-		command_buf=strdup(command);
+	if(cmd!=NULL){
+		command_buf=(char *)strdup(cmd);
 		command_name=strtok(command_buf,"!");
 		command_args=strtok(NULL,"\x0");
 	        }
@@ -684,6 +721,8 @@ void broker_contact_notification_method_data(int type, int flags, int attr, int
 		ds.state=temp_host->current_state;
 		ds.output=temp_host->plugin_output;
 	        }
+	ds.object_ptr=data;
+	ds.contact_ptr=(void *)cntct;
 	ds.ack_author=ack_author;
 	ds.ack_data=ack_data;
 	ds.escalated=escalated;
@@ -692,14 +731,14 @@ void broker_contact_notification_method_data(int type, int flags, int attr, int
 	neb_make_callbacks(NEBCALLBACK_CONTACT_NOTIFICATION_METHOD_DATA,(void *)&ds);
 
 	/* free memory */
-	free(command_buf);
+	my_free(command_buf);
 
 	return;
         }
 
 
 /* sends adaptive programs updates to broker */
-void broker_adaptive_program_data(int type, int flags, int attr, int command_type, unsigned long modhattr, unsigned long modhattrs, unsigned long modsattr, unsigned long modsattrs, char *gheh, char *gseh, struct timeval *timestamp){
+void broker_adaptive_program_data(int type, int flags, int attr, int command_type, unsigned long modhattr, unsigned long modhattrs, unsigned long modsattr, unsigned long modsattrs, struct timeval *timestamp){
 	nebstruct_adaptive_program_data ds;
 
 	if(!(event_broker_options & BROKER_ADAPTIVE_DATA))
@@ -716,8 +755,6 @@ void broker_adaptive_program_data(int type, int flags, int attr, int command_typ
 	ds.modified_host_attributes=modhattrs;
 	ds.modified_service_attribute=modsattr;
 	ds.modified_service_attributes=modsattrs;
-	ds.global_host_event_handler=gheh;
-	ds.global_service_event_handler=gseh;
 
 	/* make callbacks */
 	neb_make_callbacks(NEBCALLBACK_ADAPTIVE_PROGRAM_DATA,(void *)&ds);
@@ -776,6 +813,35 @@ void broker_adaptive_service_data(int type, int flags, int attr, service *svc, i
         }
 
 
+/* sends adaptive contact updates to broker */
+void broker_adaptive_contact_data(int type, int flags, int attr, contact *cntct, int command_type, unsigned long modattr, unsigned long modattrs, unsigned long modhattr, unsigned long modhattrs, unsigned long modsattr, unsigned long modsattrs, struct timeval *timestamp){
+	nebstruct_adaptive_contact_data ds;
+
+	if(!(event_broker_options & BROKER_ADAPTIVE_DATA))
+		return;
+
+	/* fill struct with relevant data */
+	ds.type=type;
+	ds.flags=flags;
+	ds.attr=attr;
+	ds.timestamp=get_broker_timestamp(timestamp);
+
+	ds.command_type=command_type;
+	ds.modified_attribute=modattr;
+	ds.modified_attributes=modattrs;
+	ds.modified_host_attribute=modhattr;
+	ds.modified_host_attributes=modhattrs;
+	ds.modified_service_attribute=modsattr;
+	ds.modified_service_attributes=modsattrs;
+	ds.object_ptr=(void *)cntct;
+
+	/* make callbacks */
+	neb_make_callbacks(NEBCALLBACK_ADAPTIVE_CONTACT_DATA,(void *)&ds);
+
+	return;
+        }
+
+
 /* sends external commands to broker */
 void broker_external_command(int type, int flags, int attr, int command_type, time_t entry_time, char *command_string, char *command_args, struct timeval *timestamp){
 	nebstruct_external_command_data ds;
@@ -869,6 +935,7 @@ void broker_acknowledgement_data(int type, int flags, int attr, int acknowledgem
 		ds.service_description=NULL;
 		ds.state=temp_host->current_state;
 	        }
+	ds.object_ptr=data;
 	ds.author_name=ack_author;
 	ds.comment_data=ack_data;
 	ds.is_sticky=(subtype==ACKNOWLEDGEMENT_STICKY)?TRUE:FALSE;
@@ -910,6 +977,7 @@ void broker_statechange_data(int type, int flags, int attr, int statechange_type
 		ds.service_description=NULL;
 		ds.output=temp_host->plugin_output;
 	        }
+	ds.object_ptr=data;
 	ds.state=state;
 	ds.state_type=state_type;
 	ds.current_attempt=current_attempt;
