@@ -2,8 +2,8 @@
  *
  * HISTORY.C - Nagios History CGI
  *
- * Copyright (c) 1999-2008 Ethan Galstad (egalstad@nagios.org)
- * Last Modified: 06-23-2008
+ * Copyright (c) 1999-2008 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 01-08-2008
  *
  * This CGI program will display the history for the specified host.
  * If no host is specified, the history for all hosts will be displayed.
@@ -204,9 +204,9 @@ int main(void){
 
 		printf("<table border=0 CLASS='optBox'>\n");
 		printf("<form method=\"GET\" action=\"%s\">\n",HISTORY_CGI);
-		printf("<input type='hidden' name='host' value='%s'>\n",(show_all_hosts==TRUE)?"all":escape_string(host_name));
+		printf("<input type='hidden' name='host' value='%s'>\n",(show_all_hosts==TRUE)?"all":url_encode(host_name));
 		if(display_type==DISPLAY_SERVICES)
-			printf("<input type='hidden' name='service' value='%s'>\n",escape_string(svc_description));
+			printf("<input type='hidden' name='service' value='%s'>\n",url_encode(svc_description));
 		printf("<input type='hidden' name='archive' value='%d'>\n",log_archive);
 
 		printf("<tr>\n");
@@ -489,7 +489,7 @@ void get_history(void){
 	char image[MAX_INPUT_BUFFER];
 	char image_alt[MAX_INPUT_BUFFER];
 	char *input=NULL;
-	char *input2=NULL;
+	char input_buffer2[MAX_INPUT_BUFFER];
 	char match1[MAX_INPUT_BUFFER];
 	char match2[MAX_INPUT_BUFFER];
 	int found_line=FALSE;
@@ -537,8 +537,7 @@ void get_history(void){
 
 	while(1){
 
-		my_free(input);
-		my_free(input2);
+		free(input);
 
 		if(use_lifo==TRUE){
 			if((input=pop_lifo())==NULL)
@@ -555,8 +554,7 @@ void get_history(void){
 		strcpy(image_alt,"");
 		system_message=FALSE;
 
-		if((input2=(char *)strdup(input))==NULL)
-			continue;
+		strcpy(input_buffer2,input);
 
 		/* service state alerts */
 		if(strstr(input,"SERVICE ALERT:")){
@@ -564,7 +562,7 @@ void get_history(void){
 			history_type=SERVICE_HISTORY;
 
 			/* get host and service names */
-			temp_buffer=my_strtok(input2,"]");
+			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
 			if(temp_buffer)
@@ -608,7 +606,7 @@ void get_history(void){
 			history_type=SERVICE_FLAPPING_HISTORY;
 
 			/* get host and service names */
-			temp_buffer=my_strtok(input2,"]");
+			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
 			if(temp_buffer)
@@ -640,7 +638,7 @@ void get_history(void){
 			history_type=SERVICE_DOWNTIME_HISTORY;
 
 			/* get host and service names */
-			temp_buffer=my_strtok(input2,"]");
+			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
 			if(temp_buffer)
@@ -669,7 +667,7 @@ void get_history(void){
 			history_type=HOST_HISTORY;
 
 			/* get host name */
-			temp_buffer=my_strtok(input2,"]");
+			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
 			if(temp_buffer)
@@ -703,7 +701,7 @@ void get_history(void){
 			history_type=HOST_FLAPPING_HISTORY;
 
 			/* get host name */
-			temp_buffer=my_strtok(input2,"]");
+			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
 			if(temp_buffer)
@@ -730,7 +728,7 @@ void get_history(void){
 			history_type=HOST_DOWNTIME_HISTORY;
 
 			/* get host name */
-			temp_buffer=my_strtok(input2,"]");
+			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
 			if(temp_buffer)
@@ -953,8 +951,7 @@ void get_history(void){
 
 	printf("<HR>\n");
 
-	my_free(input);
-	my_free(input2);
+	free(input);
 
 	if(use_lifo==TRUE)
 		free_lifo_memory();
