@@ -2,8 +2,8 @@
  *
  * OUTAGES.C -  Nagios Network Outages CGI
  *
- * Copyright (c) 1999-2006 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 03-21-2006
+ * Copyright (c) 1999-2008 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 01-08-2008
  *
  * License:
  * 
@@ -149,9 +149,6 @@ int main(void){
 
 	document_header(TRUE);
 
-	/* read in all host and service comments */
-	read_comment_data(get_cgi_config_location());
-
 	/* get authentication information */
 	get_authentication_information(&current_authdata);
 
@@ -225,6 +222,7 @@ void document_header(int use_stylesheet){
 
 	printf("<html>\n");
 	printf("<head>\n");
+	printf("<link rel=\"shortcut icon\" href=\"%sfavicon.ico\" type=\"image/ico\">\n",url_images_path);
 	printf("<title>\n");
 	printf("Network Outages\n");
 	printf("</title>\n");
