@@ -2,8 +2,8 @@
  *
  * DOWNTIME.C - Scheduled downtime functions for Nagios
  *
- * Copyright (c) 2000-2007 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 01-08-2007
+ * Copyright (c) 2000-2008 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 02-17-2008
  *
  * License:
  *
@@ -49,6 +49,7 @@ scheduled_downtime *scheduled_downtime_list=NULL;
 
 #ifdef NSCORE
 extern timed_event *event_list_high;
+extern timed_event *event_list_high_tail;
 #endif
 
 
@@ -62,7 +63,7 @@ extern timed_event *event_list_high;
 
 /* initializes scheduled downtime data */
 int initialize_downtime_data(char *config_file){
-	int result;
+	int result=OK;
 
 	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
 #ifdef USE_XDDDEFAULT
@@ -75,7 +76,7 @@ int initialize_downtime_data(char *config_file){
 
 /* cleans up scheduled downtime data */
 int cleanup_downtime_data(char *config_file){
-	int result;
+	int result=OK;
 	
 	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
 #ifdef USE_XDDDEFAULT
@@ -95,7 +96,9 @@ int cleanup_downtime_data(char *config_file){
 
 /* schedules a host or service downtime */
 int schedule_downtime(int type, char *host_name, char *service_description, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long *new_downtime_id){
-	unsigned long downtime_id;
+	unsigned long downtime_id=0L;
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"schedule_downtime()\n");
 
 	/* don't add old or invalid downtimes */
 	if(start_time>=end_time || end_time<=time(NULL))
@@ -117,30 +120,27 @@ int schedule_downtime(int type, char *host_name, char *service_description, time
 
 /* unschedules a host or service downtime */
 int unschedule_downtime(int type,unsigned long downtime_id){
-	scheduled_downtime *temp_downtime;
+	scheduled_downtime *temp_downtime=NULL;
 	host *hst=NULL;
 	service *svc=NULL;
-	timed_event *temp_event;
-	char temp_buffer[MAX_INPUT_BUFFER];
+	timed_event *temp_event=NULL;
 #ifdef USE_EVENT_BROKER
-	int attr;
+	int attr=0;
 #endif
-	
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"unschedule_downtime()\n");
 
 	/* find the downtime entry in the list in memory */
-	temp_downtime=find_downtime(type,downtime_id);
-	if(temp_downtime==NULL)
+	if((temp_downtime=find_downtime(type,downtime_id))==NULL)
 		return ERROR;
 
 	/* find the host or service associated with this downtime */
 	if(temp_downtime->type==HOST_DOWNTIME){
-		hst=find_host(temp_downtime->host_name);
-		if(hst==NULL)
+		if((hst=find_host(temp_downtime->host_name))==NULL)
 			return ERROR;
 	        }
 	else{
-		svc=find_service(temp_downtime->host_name,temp_downtime->service_description);
-		if(svc==NULL)
+		if((svc=find_service(temp_downtime->host_name,temp_downtime->service_description))==NULL)
 			return ERROR;
 	        }
 
@@ -168,9 +168,11 @@ int unschedule_downtime(int type,unsigned long downtime_id){
 
 			/* log a notice - this is parsed by the history CGI */
 			if(hst->scheduled_downtime_depth==0){
-				snprintf(temp_buffer,sizeof(temp_buffer),"HOST DOWNTIME ALERT: %s;CANCELLED; Scheduled downtime for host has been cancelled.\n",hst->name);
-				temp_buffer[sizeof(temp_buffer)-1]='\x0';
-				write_to_all_logs(temp_buffer,NSLOG_INFO_MESSAGE);
+
+				logit(NSLOG_INFO_MESSAGE,FALSE,"HOST DOWNTIME ALERT: %s;CANCELLED; Scheduled downtime for host has been cancelled.\n",hst->name);
+
+				/* send a notification */
+				host_notification(hst,NOTIFICATION_DOWNTIMECANCELLED,NULL,NULL,NOTIFICATION_OPTION_NONE);
 			        }
 		        }
 
@@ -181,9 +183,11 @@ int unschedule_downtime(int type,unsigned long downtime_id){
 
 			/* log a notice - this is parsed by the history CGI */
 			if(svc->scheduled_downtime_depth==0){
-				snprintf(temp_buffer,sizeof(temp_buffer),"SERVICE DOWNTIME ALERT: %s;%s;CANCELLED; Scheduled downtime for service has been cancelled.\n",svc->host_name,svc->description);
-				temp_buffer[sizeof(temp_buffer)-1]='\x0';
-				write_to_all_logs(temp_buffer,NSLOG_INFO_MESSAGE);
+
+				logit(NSLOG_INFO_MESSAGE,FALSE,"SERVICE DOWNTIME ALERT: %s;%s;CANCELLED; Scheduled downtime for service has been cancelled.\n",svc->host_name,svc->description);
+
+				/* send a notification */
+				service_notification(svc,NOTIFICATION_DOWNTIMECANCELLED,NULL,NULL,NOTIFICATION_OPTION_NONE);
 			        }
 		        }
 	        }
@@ -196,7 +200,7 @@ int unschedule_downtime(int type,unsigned long downtime_id){
 			break;
 	        }
 	if(temp_event!=NULL)
-		remove_event(temp_event,&event_list_high);
+		remove_event(temp_event,&event_list_high,&event_list_high_tail);
 
 	/* delete downtime entry */
 	if(temp_downtime->type==HOST_DOWNTIME)
@@ -205,10 +209,18 @@ int unschedule_downtime(int type,unsigned long downtime_id){
 		delete_service_downtime(downtime_id);
 
 	/* unschedule all downtime entries that were triggered by this one */
-	for(temp_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=temp_downtime->next){
-		if(temp_downtime->triggered_by==downtime_id)
-			unschedule_downtime(ANY_DOWNTIME,temp_downtime->downtime_id);
-	        }
+	while(1){
+
+		for(temp_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=temp_downtime->next){
+			if(temp_downtime->triggered_by==downtime_id){
+				unschedule_downtime(ANY_DOWNTIME,temp_downtime->downtime_id);
+				break;
+				}
+			}
+
+		if(temp_downtime==NULL)
+			break;
+		}
 
 	return OK;
         }
@@ -217,15 +229,19 @@ int unschedule_downtime(int type,unsigned long downtime_id){
 
 /* registers scheduled downtime (schedules it, adds comments, etc.) */
 int register_downtime(int type, unsigned long downtime_id){
- 	char temp_buffer[MAX_INPUT_BUFFER];
-	char start_time_string[MAX_DATETIME_LENGTH];
-	char end_time_string[MAX_DATETIME_LENGTH];
-	scheduled_downtime *temp_downtime;
+ 	char *temp_buffer=NULL;
+	char start_time_string[MAX_DATETIME_LENGTH]="";
+	char end_time_string[MAX_DATETIME_LENGTH]="";
+	scheduled_downtime *temp_downtime=NULL;
 	host *hst=NULL;
 	service *svc=NULL;
-	char *type_string="";
-	int hours;
-	int minutes;
+	char *type_string=NULL;
+	int hours=0;
+	int minutes=0;
+	int seconds=0;
+	unsigned long *new_downtime_id=NULL;
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"register_downtime()\n");
 
 	/* find the downtime entry in memory */
 	temp_downtime=find_downtime(type,downtime_id);
@@ -234,13 +250,11 @@ int register_downtime(int type, unsigned long downtime_id){
 
 	/* find the host or service associated with this downtime */
 	if(temp_downtime->type==HOST_DOWNTIME){
-		hst=find_host(temp_downtime->host_name);
-		if(hst==NULL)
+		if((hst=find_host(temp_downtime->host_name))==NULL)
 			return ERROR;
 	        }
 	else{
-		svc=find_service(temp_downtime->host_name,temp_downtime->service_description);
-		if(svc==NULL)
+		if((svc=find_service(temp_downtime->host_name,temp_downtime->service_description))==NULL)
 			return ERROR;
 	        }
 
@@ -249,17 +263,33 @@ int register_downtime(int type, unsigned long downtime_id){
 	get_datetime_string(&(temp_downtime->end_time),end_time_string,MAX_DATETIME_LENGTH,SHORT_DATE_TIME);
 	hours=temp_downtime->duration/3600;
 	minutes=((temp_downtime->duration-(hours*3600))/60);
+	seconds=temp_downtime->duration-(hours*3600)-(minutes*60);
 	if(temp_downtime->type==HOST_DOWNTIME)
 		type_string="host";
 	else
 		type_string="service";
-	if(temp_downtime->fixed==TRUE){
-		snprintf(temp_buffer,sizeof(temp_buffer)-1,"This %s has been scheduled for fixed downtime from %s to %s.  Notifications for the %s will not be sent out during that time period.",type_string,start_time_string,end_time_string,type_string);
-	        }
+	if(temp_downtime->fixed==TRUE)
+		asprintf(&temp_buffer,"This %s has been scheduled for fixed downtime from %s to %s.  Notifications for the %s will not be sent out during that time period.",type_string,start_time_string,end_time_string,type_string);
+	else
+		asprintf(&temp_buffer,"This %s has been scheduled for flexible downtime starting between %s and %s and lasting for a period of %d hours and %d minutes.  Notifications for the %s will not be sent out during that time period.",type_string,start_time_string,end_time_string,hours,minutes,type_string);
+
+
+	log_debug_info(DEBUGL_DOWNTIME,0,"Scheduled Downtime Details:\n");
+	if(temp_downtime->type==HOST_DOWNTIME){
+		log_debug_info(DEBUGL_DOWNTIME,0," Type:        Host Downtime\n");
+		log_debug_info(DEBUGL_DOWNTIME,0," Host:        %s\n",hst->name);
+		}
 	else{
-		snprintf(temp_buffer,sizeof(temp_buffer)-1,"This %s has been scheduled for flexible downtime starting between %s and %s and lasting for a period of %d hours and %d minutes.  Notifications for the %s will not be sent out during that time period.",type_string,start_time_string,end_time_string,hours,minutes,type_string);
-	        }
-	temp_buffer[sizeof(temp_buffer)-1]='\x0';
+		log_debug_info(DEBUGL_DOWNTIME,0," Type:        Service Downtime\n");
+		log_debug_info(DEBUGL_DOWNTIME,0," Host:        %s\n",svc->host_name);
+		log_debug_info(DEBUGL_DOWNTIME,0," Service:     %s\n",svc->description);
+		}
+	log_debug_info(DEBUGL_DOWNTIME,0," Fixed/Flex:  %s\n",(temp_downtime->fixed==TRUE)?"Fixed":"Flexible");
+	log_debug_info(DEBUGL_DOWNTIME,0," Start:       %lu\n",temp_downtime->downtime_id);
+	log_debug_info(DEBUGL_DOWNTIME,0," End:         %lu\n",temp_downtime->downtime_id);
+	log_debug_info(DEBUGL_DOWNTIME,0," Duration:    %dh %dm %ds\n",hours,minutes,seconds);
+	log_debug_info(DEBUGL_DOWNTIME,0," Downtime ID: %lu\n",temp_downtime->downtime_id);
+	log_debug_info(DEBUGL_DOWNTIME,0," Trigger ID:  %lu\n",temp_downtime->triggered_by);
 
 
 	/* add a non-persistent comment to the host or service regarding the scheduled outage */
@@ -272,8 +302,25 @@ int register_downtime(int type, unsigned long downtime_id){
 	/*** SCHEDULE DOWNTIME - FLEXIBLE (NON-FIXED) DOWNTIME IS HANDLED AT A LATER POINT ***/
 
 	/* only non-triggered downtime is scheduled... */
-	if(temp_downtime->triggered_by==0)
-		schedule_new_event(EVENT_SCHEDULED_DOWNTIME,TRUE,temp_downtime->start_time,FALSE,0,NULL,FALSE,(void *)temp_downtime->downtime_id,NULL);
+	if(temp_downtime->triggered_by==0){
+		if((new_downtime_id=(unsigned long *)malloc(sizeof(unsigned long *)))){
+			*new_downtime_id=downtime_id;
+			schedule_new_event(EVENT_SCHEDULED_DOWNTIME,TRUE,temp_downtime->start_time,FALSE,0,NULL,FALSE,(void *)new_downtime_id,NULL,0);
+			}
+		}
+
+#ifdef PROBABLY_NOT_NEEDED
+	/*** FLEXIBLE DOWNTIME SANITY CHECK - ADDED 02/17/2008 ****/
+
+	/* if host/service is in a non-OK/UP state right now, see if we should start flexible time immediately */
+	/* this is new logic added in 3.0rc3 */
+	if(temp_downtime->fixed==FALSE){
+		if(temp_downtime->type==HOST_DOWNTIME)
+			check_pending_flex_host_downtime(hst);
+		else
+			check_pending_flex_service_downtime(svc);
+		}
+#endif
 
 	return OK;
         }
@@ -296,35 +343,32 @@ int handle_scheduled_downtime_by_id(unsigned long downtime_id){
 
 /* handles scheduled host or service downtime */
 int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
- 	char buffer[MAX_INPUT_BUFFER];
-	scheduled_downtime *this_downtime;
+	scheduled_downtime *this_downtime=NULL;
 	host *hst=NULL;
 	service *svc=NULL;
-	time_t event_time;
+	time_t event_time=0L;
+	unsigned long *new_downtime_id=NULL;
 #ifdef USE_EVENT_BROKER
-	int attr;
+	int attr=0;
 #endif
 
-#ifdef DEBUG0
-        printf("handle_scheduled_downtime() start\n");
-#endif
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"handle_scheduled_downtime()\n");
 
 	if(temp_downtime==NULL)
 		return ERROR;
 
 	/* find the host or service associated with this downtime */
 	if(temp_downtime->type==HOST_DOWNTIME){
-		hst=find_host(temp_downtime->host_name);
-		if(hst==NULL)
+		if((hst=find_host(temp_downtime->host_name))==NULL)
 			return ERROR;
 	        }
 	else{
-		svc=find_service(temp_downtime->host_name,temp_downtime->service_description);
-		if(svc==NULL)
+		if((svc=find_service(temp_downtime->host_name,temp_downtime->service_description))==NULL)
 			return ERROR;
 	        }
 
-	/* if downtime if flexible and host/svc is in an ok state, don't do anything right now (wait for event handler to kill it off) */
+	/* if downtime if flexible and host/svc is in an ok state, don't do anything right now (wait for event handler to kick it off) */
 	/* start_flex_downtime variable is set to TRUE by event handler functions */
 	if(temp_downtime->fixed==FALSE){
 
@@ -343,7 +387,7 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 
 				/*** SINCE THE FLEX DOWNTIME MAY NEVER START, WE HAVE TO PROVIDE A WAY OF EXPIRING UNUSED DOWNTIME... ***/
 
-				schedule_new_event(EVENT_EXPIRE_DOWNTIME,TRUE,(temp_downtime->end_time+1),FALSE,0,NULL,FALSE,NULL,NULL);
+				schedule_new_event(EVENT_EXPIRE_DOWNTIME,TRUE,(temp_downtime->end_time+1),FALSE,0,NULL,FALSE,NULL,NULL,0);
 
 				return OK;
 			        }
@@ -367,18 +411,24 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 
 		if(temp_downtime->type==HOST_DOWNTIME && hst->scheduled_downtime_depth==0){
 
+			log_debug_info(DEBUGL_DOWNTIME,0,"Host '%s' has exited from a period of scheduled downtime (id=%lu).\n",hst->name,temp_downtime->downtime_id);
+
 			/* log a notice - this one is parsed by the history CGI */
-			snprintf(buffer,sizeof(buffer),"HOST DOWNTIME ALERT: %s;STOPPED; Host has exited from a period of scheduled downtime",hst->name);
-			buffer[sizeof(buffer)-1]='\x0';
-			write_to_all_logs(buffer,NSLOG_INFO_MESSAGE);
+			logit(NSLOG_INFO_MESSAGE,FALSE,"HOST DOWNTIME ALERT: %s;STOPPED; Host has exited from a period of scheduled downtime",hst->name);
+
+			/* send a notification */
+			host_notification(hst,NOTIFICATION_DOWNTIMEEND,temp_downtime->author,temp_downtime->comment,NOTIFICATION_OPTION_NONE);
 		        }
 
 		else if(temp_downtime->type==SERVICE_DOWNTIME && svc->scheduled_downtime_depth==0){
 
+			log_debug_info(DEBUGL_DOWNTIME,0,"Service '%s' on host '%s' has exited from a period of scheduled downtime (id=%lu).\n",svc->description,svc->host_name,temp_downtime->downtime_id);
+
 			/* log a notice - this one is parsed by the history CGI */
-			snprintf(buffer,sizeof(buffer),"SERVICE DOWNTIME ALERT: %s;%s;STOPPED; Service has exited from a period of scheduled downtime",svc->host_name,svc->description);
-			buffer[sizeof(buffer)-1]='\x0';
-			write_to_all_logs(buffer,NSLOG_INFO_MESSAGE);
+			logit(NSLOG_INFO_MESSAGE,FALSE,"SERVICE DOWNTIME ALERT: %s;%s;STOPPED; Service has exited from a period of scheduled downtime",svc->host_name,svc->description);
+
+			/* send a notification */
+			service_notification(svc,NOTIFICATION_DOWNTIMEEND,temp_downtime->author,temp_downtime->comment,NOTIFICATION_OPTION_NONE);
 		        }
 
 
@@ -401,12 +451,21 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 		        }
 
 		/* handle (stop) downtime that is triggered by this one */
-		for(this_downtime=scheduled_downtime_list;this_downtime!=NULL;this_downtime=this_downtime->next){
-			if(this_downtime->triggered_by==temp_downtime->downtime_id)
-				handle_scheduled_downtime(this_downtime);
-		        }
+		while(1){
 
-		/* delete downtime entry from the log */
+			/* list contents might change by recursive calls, so we use this inefficient method to prevent segfaults */
+			for(this_downtime=scheduled_downtime_list;this_downtime!=NULL;this_downtime=this_downtime->next){
+				if(this_downtime->triggered_by==temp_downtime->downtime_id){
+					handle_scheduled_downtime(this_downtime);
+					break;
+					}
+				}
+
+			if(this_downtime==NULL)
+				break;
+			}
+
+		/* delete downtime entry */
 		if(temp_downtime->type==HOST_DOWNTIME)
 			delete_host_downtime(temp_downtime->downtime_id);
 		else
@@ -423,18 +482,24 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 
 		if(temp_downtime->type==HOST_DOWNTIME && hst->scheduled_downtime_depth==0){
 
+			log_debug_info(DEBUGL_DOWNTIME,0,"Host '%s' has entered a period of scheduled downtime (id=%lu).\n",hst->name,temp_downtime->downtime_id);
+
 			/* log a notice - this one is parsed by the history CGI */
-			snprintf(buffer,sizeof(buffer),"HOST DOWNTIME ALERT: %s;STARTED; Host has entered a period of scheduled downtime",hst->name);
-			buffer[sizeof(buffer)-1]='\x0';
-			write_to_all_logs(buffer,NSLOG_INFO_MESSAGE);
+			logit(NSLOG_INFO_MESSAGE,FALSE,"HOST DOWNTIME ALERT: %s;STARTED; Host has entered a period of scheduled downtime",hst->name);
+
+			/* send a notification */
+			host_notification(hst,NOTIFICATION_DOWNTIMESTART,temp_downtime->author,temp_downtime->comment,NOTIFICATION_OPTION_NONE);
 		        }
 
 		else if(temp_downtime->type==SERVICE_DOWNTIME && svc->scheduled_downtime_depth==0){
 
+			log_debug_info(DEBUGL_DOWNTIME,0,"Service '%s' on host '%s' has entered a period of scheduled downtime (id=%lu).\n",svc->description,svc->host_name,temp_downtime->downtime_id);
+
 			/* log a notice - this one is parsed by the history CGI */
-			snprintf(buffer,sizeof(buffer),"SERVICE DOWNTIME ALERT: %s;%s;STARTED; Service has entered a period of scheduled downtime",svc->host_name,svc->description);
-			buffer[sizeof(buffer)-1]='\x0';
-			write_to_all_logs(buffer,NSLOG_INFO_MESSAGE);
+			logit(NSLOG_INFO_MESSAGE,FALSE,"SERVICE DOWNTIME ALERT: %s;%s;STARTED; Service has entered a period of scheduled downtime",svc->host_name,svc->description);
+
+			/* send a notification */
+			service_notification(svc,NOTIFICATION_DOWNTIMESTART,temp_downtime->author,temp_downtime->comment,NOTIFICATION_OPTION_NONE);
 		        }
 
 		/* increment the downtime depth variable */
@@ -457,7 +522,10 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 			event_time=(time_t)((unsigned long)time(NULL)+temp_downtime->duration);
 		else
 			event_time=temp_downtime->end_time;
-		schedule_new_event(EVENT_SCHEDULED_DOWNTIME,TRUE,event_time,FALSE,0,NULL,FALSE,(void *)temp_downtime->downtime_id,NULL);
+		if((new_downtime_id=(unsigned long *)malloc(sizeof(unsigned long *)))){
+			*new_downtime_id=temp_downtime->downtime_id;
+			schedule_new_event(EVENT_SCHEDULED_DOWNTIME,TRUE,event_time,FALSE,0,NULL,FALSE,(void *)new_downtime_id,NULL,0);
+			}
 
 		/* handle (start) downtime that is triggered by this one */
 		for(this_downtime=scheduled_downtime_list;this_downtime!=NULL;this_downtime=this_downtime->next){
@@ -466,10 +534,6 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 		        }
 	        }
 	
-#ifdef DEBUG0
-	printf("handle_scheduled_downtime() end\n");
-#endif
-	
 	return OK;
         }
 
@@ -477,12 +541,11 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 
 /* checks for flexible (non-fixed) host downtime that should start now */
 int check_pending_flex_host_downtime(host *hst){
-	scheduled_downtime *temp_downtime;
-	time_t current_time;
+	scheduled_downtime *temp_downtime=NULL;
+	time_t current_time=0L;
 
-#ifdef DEBUG0
-	printf("check_pending_flex_host_downtime() start\n");
-#endif
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"check_pending_flex_host_downtime()\n");
 
 	if(hst==NULL)
 		return ERROR;
@@ -514,28 +577,26 @@ int check_pending_flex_host_downtime(host *hst){
 			
 			/* if the time boundaries are okay, start this scheduled downtime */
 			if(temp_downtime->start_time<=current_time && current_time<=temp_downtime->end_time){
+
+				log_debug_info(DEBUGL_DOWNTIME,0,"Flexible downtime (id=%lu) for host '%s' starting now...\n",temp_downtime->downtime_id,hst->name);
+
 				temp_downtime->start_flex_downtime=TRUE;
 				handle_scheduled_downtime(temp_downtime);
 			        }
 		        }
 	        }
 
-#ifdef DEBUG0
-	printf("check_pending_flex_host_downtime() end\n");
-#endif
-
 	return OK;
         }
 
 
 /* checks for flexible (non-fixed) service downtime that should start now */
 int check_pending_flex_service_downtime(service *svc){
-	scheduled_downtime *temp_downtime;
-	time_t current_time;
+	scheduled_downtime *temp_downtime=NULL;
+	time_t current_time=0L;
 
-#ifdef DEBUG0
-	printf("check_pending_flex_service_downtime() start\n");
-#endif
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"check_pending_flex_service_downtime()\n");
 
 	if(svc==NULL)
 		return ERROR;
@@ -567,29 +628,27 @@ int check_pending_flex_service_downtime(service *svc){
 
 			/* if the time boundaries are okay, start this scheduled downtime */
 			if(temp_downtime->start_time<=current_time && current_time<=temp_downtime->end_time){
+
+				log_debug_info(DEBUGL_DOWNTIME,0,"Flexible downtime (id=%lu) for service '%s' on host '%s' starting now...\n",temp_downtime->downtime_id,svc->description,svc->host_name);
+
 				temp_downtime->start_flex_downtime=TRUE;
 				handle_scheduled_downtime(temp_downtime);
 			        }
 		        }
 	        }
 
-#ifdef DEBUG0
-	printf("check_pending_flex_service_downtime() end\n");
-#endif
-
 	return OK;
         }
 
 
 /* checks for (and removes) expired downtime entries */
 int check_for_expired_downtime(void){
-	scheduled_downtime *temp_downtime;
-	scheduled_downtime *next_downtime;
-	time_t current_time;
+	scheduled_downtime *temp_downtime=NULL;
+	scheduled_downtime *next_downtime=NULL;
+	time_t current_time=0L;
 
-#ifdef DEBUG0
-	printf("check_for_expired_downtime() start\n");
-#endif
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"check_for_expired_downtime()\n");
 
 	time(&current_time);
 
@@ -601,6 +660,8 @@ int check_for_expired_downtime(void){
 		/* this entry should be removed */
 		if(temp_downtime->is_in_effect==FALSE && temp_downtime->end_time<current_time){
 
+			log_debug_info(DEBUGL_DOWNTIME,0,"Expiring %s downtime (id=%lu)...\n",(temp_downtime->type==HOST_DOWNTIME)?"host":"service",temp_downtime->downtime_id);
+
 			/* delete the downtime entry */
 			if(temp_downtime->type==HOST_DOWNTIME)
 				delete_host_downtime(temp_downtime->downtime_id);
@@ -609,10 +670,6 @@ int check_for_expired_downtime(void){
 		        }
 	        }
 
-#ifdef DEBUG0
-	printf("check_for_expired_downtime() end\n");
-#endif
-
 	return OK;
         }
 
@@ -625,7 +682,7 @@ int check_for_expired_downtime(void){
 
 /* save a host or service downtime */
 int add_new_downtime(int type, char *host_name, char *service_description, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long *downtime_id){
-	int result;
+	int result=OK;
 
 	if(type==HOST_DOWNTIME)
 		result=add_new_host_downtime(host_name,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id);
@@ -638,8 +695,8 @@ int add_new_downtime(int type, char *host_name, char *service_description, time_
 
 /* saves a host downtime entry */
 int add_new_host_downtime(char *host_name, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long *downtime_id){
-	int result;
-	unsigned long new_downtime_id;
+	int result=OK;
+	unsigned long new_downtime_id=0L;
 
 	if(host_name==NULL)
 		return ERROR;
@@ -664,8 +721,8 @@ int add_new_host_downtime(char *host_name, time_t entry_time, char *author, char
 
 /* saves a service downtime entry */
 int add_new_service_downtime(char *host_name, char *service_description, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long *downtime_id){
-	int result;
-	unsigned long new_downtime_id;
+	int result=OK;
+	unsigned long new_downtime_id=0L;
 
 	if(host_name==NULL || service_description==NULL)
 		return ERROR;
@@ -696,7 +753,7 @@ int add_new_service_downtime(char *host_name, char *service_description, time_t
 
 /* deletes a scheduled host or service downtime entry from the list in memory */
 int delete_downtime(int type,unsigned long downtime_id){
-	int result;
+	int result=OK;
 	scheduled_downtime *this_downtime=NULL;
 	scheduled_downtime *last_downtime=NULL;
 	scheduled_downtime *next_downtime=NULL;
@@ -732,11 +789,11 @@ int delete_downtime(int type,unsigned long downtime_id){
 			last_downtime->next=next_downtime;
 		
 		/* free memory */
-		free(this_downtime->host_name);
-		free(this_downtime->service_description);
-		free(this_downtime->author);
-		free(this_downtime->comment);
-		free(this_downtime);
+		my_free(this_downtime->host_name);
+		my_free(this_downtime->service_description);
+		my_free(this_downtime->author);
+		my_free(this_downtime->comment);
+		my_free(this_downtime);
 
 		result=OK;
 	        }
@@ -749,7 +806,7 @@ int delete_downtime(int type,unsigned long downtime_id){
 
 /* deletes a scheduled host downtime entry */
 int delete_host_downtime(unsigned long downtime_id){
-	int result;
+	int result=OK;
 
 	/* delete the downtime from memory */
 	delete_downtime(HOST_DOWNTIME,downtime_id);
@@ -765,7 +822,7 @@ int delete_host_downtime(unsigned long downtime_id){
 
 /* deletes a scheduled service downtime entry */
 int delete_service_downtime(unsigned long downtime_id){
-	int result;
+	int result=OK;
 
 	/* delete the downtime from memory */
 	delete_downtime(SERVICE_DOWNTIME,downtime_id);
@@ -782,26 +839,6 @@ int delete_service_downtime(unsigned long downtime_id){
 
 
 
-#ifdef NSCGI
-
-/******************************************************************/
-/************************ INPUT FUNCTIONS *************************/
-/******************************************************************/
-
-/* reads all downtime data */
-int read_downtime_data(char *main_config_file){
-	int result;
-
-	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
-#ifdef USE_XDDDEFAULT
-	result=xdddefault_read_downtime_data(main_config_file);
-#endif
-
-	return result;
-        }
-
-#endif
-
 
 
 /******************************************************************/
@@ -810,7 +847,7 @@ int read_downtime_data(char *main_config_file){
 
 /* adds a host downtime entry to the list in memory */
 int add_host_downtime(char *host_name, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long downtime_id){
-	int result;
+	int result=OK;
 
 	result=add_downtime(HOST_DOWNTIME,host_name,NULL,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id);
 
@@ -820,7 +857,7 @@ int add_host_downtime(char *host_name, time_t entry_time, char *author, char *co
 
 /* adds a service downtime entry to the list in memory */
 int add_service_downtime(char *host_name, char *svc_description, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long downtime_id){
-	int result;
+	int result=OK;
 
 	result=add_downtime(SERVICE_DOWNTIME,host_name,svc_description,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id);
 
@@ -833,42 +870,51 @@ int add_downtime(int downtime_type, char *host_name, char *svc_description, time
 	scheduled_downtime *new_downtime=NULL;
 	scheduled_downtime *last_downtime=NULL;
 	scheduled_downtime *temp_downtime=NULL;
+	int result=OK;
 
 	/* don't add triggered downtimes that don't have a valid parent */
 	if(triggered_by>0  && find_downtime(ANY_DOWNTIME,triggered_by)==NULL)
 		return ERROR;
 
+	/* we don't have enough info */
+	if(host_name==NULL || (downtime_type==SERVICE_DOWNTIME && svc_description==NULL))
+		return ERROR;
+
 	/* allocate memory for the downtime */
-	new_downtime=(scheduled_downtime *)malloc(sizeof(scheduled_downtime));
-	if(new_downtime==NULL)
+	if((new_downtime=(scheduled_downtime *)malloc(sizeof(scheduled_downtime)))==NULL)
 		return ERROR;
 
-	new_downtime->host_name=strdup(host_name);
-	if(new_downtime->host_name==NULL){
-		free(new_downtime);
-		return ERROR;
-	        }
+	/* initialize vars */
+	new_downtime->host_name=NULL;
+	new_downtime->service_description=NULL;
+	new_downtime->author=NULL;
+	new_downtime->comment=NULL;
 
+	/* duplicate vars */
+	if((new_downtime->host_name=(char *)strdup(host_name))==NULL)
+		result=ERROR;
 	if(downtime_type==SERVICE_DOWNTIME){
-		new_downtime->service_description=strdup(svc_description);
-		if(new_downtime->service_description==NULL){
-			free(new_downtime->host_name);
-			free(new_downtime);
-			return ERROR;
-		        }
+		if((new_downtime->service_description=(char *)strdup(svc_description))==NULL)
+			result=ERROR;
+	        }
+	if(author){
+		if((new_downtime->author=(char *)strdup(author))==NULL)
+			result=ERROR;
+	        }
+	if(comment_data){
+		if((new_downtime->comment=(char *)strdup(comment_data))==NULL)
+			result=ERROR;
 	        }
-	else
-		new_downtime->service_description=NULL;
 
-	if(author==NULL)
-		new_downtime->author=NULL;
-	else
-		new_downtime->author=strdup(author);
-
-	if(comment_data==NULL)
-		new_downtime->comment=NULL;
-	else
-		new_downtime->comment=strdup(comment_data);
+	/* handle errors */
+	if(result==ERROR){
+		my_free(new_downtime->comment);
+		my_free(new_downtime->author);
+		my_free(new_downtime->service_description);
+		my_free(new_downtime->host_name);
+		my_free(new_downtime);
+		return ERROR;
+	        }
 
 	new_downtime->type=downtime_type;
 	new_downtime->entry_time=entry_time;
@@ -885,7 +931,6 @@ int add_downtime(int downtime_type, char *host_name, char *svc_description, time
 	new_downtime->incremented_pending_downtime=FALSE;
 #endif
 
-
 	/* add new downtime to downtime list, sorted by start time */
 	last_downtime=scheduled_downtime_list;
 	for(temp_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=temp_downtime->next){
@@ -928,7 +973,7 @@ int add_downtime(int downtime_type, char *host_name, char *svc_description, time
 
 /* finds a specific downtime entry */
 scheduled_downtime *find_downtime(int type, unsigned long downtime_id){
-	scheduled_downtime *temp_downtime;
+	scheduled_downtime *temp_downtime=NULL;
 
 	for(temp_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=temp_downtime->next){
 		if(type!=ANY_DOWNTIME && temp_downtime->type!=type)
@@ -962,17 +1007,17 @@ scheduled_downtime *find_service_downtime(unsigned long downtime_id){
 
 /* frees memory allocated for the scheduled downtime data */
 void free_downtime_data(void){
-	scheduled_downtime *this_downtime;
-	scheduled_downtime *next_downtime;
+	scheduled_downtime *this_downtime=NULL;
+	scheduled_downtime *next_downtime=NULL;
 
 	/* free memory for the scheduled_downtime list */
 	for(this_downtime=scheduled_downtime_list;this_downtime!=NULL;this_downtime=next_downtime){
 		next_downtime=this_downtime->next;
-		free(this_downtime->host_name);
-		free(this_downtime->service_description);
-		free(this_downtime->author);
-		free(this_downtime->comment);
-		free(this_downtime);
+		my_free(this_downtime->host_name);
+		my_free(this_downtime->service_description);
+		my_free(this_downtime->author);
+		my_free(this_downtime->comment);
+		my_free(this_downtime);
 	        }
 
 	/* reset list pointer */
