@@ -2,8 +2,8 @@
  *
  * OBJECTS.H - Header file for object addition/search functions
  *
- * Copyright (c) 1999-2006 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 03-21-2006
+ * Copyright (c) 1999-2007 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 11-10-2007
  *
  * License:
  *
@@ -37,44 +37,49 @@
 
 /*************** CURRENT OBJECT REVISION **************/
 
-#define CURRENT_OBJECT_STRUCTURE_VERSION        2
+#define CURRENT_OBJECT_STRUCTURE_VERSION        307     /* increment when changes are made to data structures... */
+	                                                /* Nagios 3 starts at 300, Nagios 4 at 400, etc. */
 
 
 
 /***************** OBJECT SIZE LIMITS *****************/
 
-#define MAX_HOSTNAME_LENGTH            		64	/* max. host name length */
-#define MAX_SERVICEDESC_LENGTH			64	/* max. service description length */
-#define MAX_PLUGINOUTPUT_LENGTH			332	/* max. length of plugin output */
-
 #define MAX_STATE_HISTORY_ENTRIES		21	/* max number of old states to keep track of for flap detection */
-
 #define MAX_CONTACT_ADDRESSES                   6       /* max number of custom addresses a contact can have */
 
 
 
-/***************** CHAINED HASH LIMITS ****************/
+/***************** SKIP LISTS ****************/
 
-#define SERVICE_HASHSLOTS                      1024
-#define HOST_HASHSLOTS                         1024
-#define COMMAND_HASHSLOTS                      256
-#define TIMEPERIOD_HASHSLOTS                   64
-#define CONTACT_HASHSLOTS                      128
-#define CONTACTGROUP_HASHSLOTS                 64
-#define HOSTGROUP_HASHSLOTS                    128
-#define SERVICEGROUP_HASHSLOTS                 128
-#define HOSTEXTINFO_HASHSLOTS                  1024
-#define SERVICEEXTINFO_HASHSLOTS               1024
-
-#define HOSTDEPENDENCY_HASHSLOTS               1024
-#define SERVICEDEPENDENCY_HASHSLOTS            1024
-#define HOSTESCALATION_HASHSLOTS               1024
-#define SERVICEESCALATION_HASHSLOTS            1024
+#define NUM_OBJECT_SKIPLISTS                   12
 
+#define HOST_SKIPLIST                          0
+#define SERVICE_SKIPLIST                       1
+#define COMMAND_SKIPLIST                       2
+#define TIMEPERIOD_SKIPLIST                    3
+#define CONTACT_SKIPLIST                       4
+#define CONTACTGROUP_SKIPLIST                  5
+#define HOSTGROUP_SKIPLIST                     6
+#define SERVICEGROUP_SKIPLIST                  7
+#define HOSTDEPENDENCY_SKIPLIST                8
+#define SERVICEDEPENDENCY_SKIPLIST             9
+#define HOSTESCALATION_SKIPLIST                10
+#define SERVICEESCALATION_SKIPLIST             11
 
 
 /****************** DATA STRUCTURES *******************/
 
+typedef struct host_struct host;
+typedef struct service_struct service;
+typedef struct contact_struct contact;
+
+/* OBJECT LIST STRUCTURE */
+typedef struct objectlist_struct{
+	void      *object_ptr;
+	struct objectlist_struct *next;
+        }objectlist;
+
+
 /* TIMERANGE structure */
 typedef struct timerange_struct{
 	unsigned long range_start;
@@ -83,28 +88,60 @@ typedef struct timerange_struct{
         }timerange;
 
 
+/* DATERANGE structure */
+typedef struct daterange_struct{
+	int type;
+	int syear;          /* start year */
+	int smon;           /* start month */
+	int smday;          /* start day of month (may 3rd, last day in feb) */
+	int swday;          /* start day of week (thursday) */
+	int swday_offset;   /* start weekday offset (3rd thursday, last monday in jan) */
+	int eyear;
+	int emon;
+	int emday;
+	int ewday;
+	int ewday_offset;
+	int skip_interval;
+	timerange *times;
+	struct daterange_struct *next;
+	}daterange;
+
+
+/* TIMEPERIODEXCLUSION structure */
+typedef struct timeperiodexclusion_struct{
+	char  *timeperiod_name;
+	struct timeperiod_struct *timeperiod_ptr;
+	struct timeperiodexclusion_struct *next;
+        }timeperiodexclusion;
+
+
 /* TIMEPERIOD structure */
 typedef struct timeperiod_struct{
 	char    *name;
 	char    *alias;
 	timerange *days[7];
+	daterange *exceptions[DATERANGE_TYPES];
+	timeperiodexclusion *exclusions;
 	struct 	timeperiod_struct *next;
 	struct 	timeperiod_struct *nexthash;
 	}timeperiod;
 
 
-/* CONTACTGROUPMEMBER structure */
-typedef struct contactgroupmember_struct{
+/* CONTACTSMEMBER structure */
+typedef struct contactsmember_struct{
 	char    *contact_name;
-	struct  contactgroupmember_struct *next;
-        }contactgroupmember;
+#ifdef NSCORE
+	contact *contact_ptr;
+#endif
+	struct  contactsmember_struct *next;
+        }contactsmember;
 
 
 /* CONTACTGROUP structure */
 typedef struct contactgroup_struct{
 	char	*group_name;
 	char    *alias;
-	contactgroupmember *members;
+	contactsmember *members;
 	struct	contactgroup_struct *next;
 	struct	contactgroup_struct *nexthash;
 	}contactgroup;
@@ -112,39 +149,152 @@ typedef struct contactgroup_struct{
 
 /* CONTACTGROUPSMEMBER structure */
 typedef struct contactgroupsmember_struct{
-	char *group_name;
+	char    *group_name;
+#ifdef NSCORE
+	contactgroup *group_ptr;
+#endif
 	struct contactgroupsmember_struct *next;
         }contactgroupsmember;
 
 
+/* CUSTOMVARIABLESMEMBER structure */
+typedef struct customvariablesmember_struct{
+	char    *variable_name;
+	char    *variable_value;
+	int     has_been_modified;
+	struct customvariablesmember_struct *next;
+        }customvariablesmember;
+
+
+/* COMMAND structure */
+typedef struct command_struct{
+	char    *name;
+	char    *command_line;
+	struct command_struct *next;
+	struct command_struct *nexthash;
+        }command;
+
+
+/* COMMANDSMEMBER structure */
+typedef struct commandsmember_struct{
+	char	*command;
+#ifdef NSCORE
+	command *command_ptr;
+#endif
+	struct	commandsmember_struct *next;
+	}commandsmember;
+
+
+/* CONTACT structure */
+struct contact_struct{
+	char	*name;
+	char	*alias;
+	char	*email;
+	char	*pager;
+	char    *address[MAX_CONTACT_ADDRESSES];
+	commandsmember *host_notification_commands;
+	commandsmember *service_notification_commands;	
+	int     notify_on_service_unknown;
+	int     notify_on_service_warning;
+	int     notify_on_service_critical;
+	int     notify_on_service_recovery;
+	int     notify_on_service_flapping;
+	int     notify_on_service_downtime;
+	int 	notify_on_host_down;
+	int	notify_on_host_unreachable;
+	int	notify_on_host_recovery;
+	int     notify_on_host_flapping;
+	int     notify_on_host_downtime;
+	char	*host_notification_period;
+	char	*service_notification_period;
+	int     host_notifications_enabled;
+	int     service_notifications_enabled;
+	int     can_submit_commands;
+	int     retain_status_information;
+	int     retain_nonstatus_information;
+	customvariablesmember *custom_variables;
+#ifdef NSCORE
+	time_t  last_host_notification;
+	time_t  last_service_notification;
+	unsigned long modified_attributes;
+	unsigned long modified_host_attributes;
+	unsigned long modified_service_attributes;
+
+	timeperiod *host_notification_period_ptr;
+	timeperiod *service_notification_period_ptr;
+	objectlist *contactgroups_ptr;
+#endif
+	struct	contact_struct *next;
+	struct	contact_struct *nexthash;
+        };
+
+
+/* SERVICESMEMBER structure */
+typedef struct servicesmember_struct{
+	char    *host_name;
+	char    *service_description;
+#ifdef NSCORE
+	service *service_ptr;
+#endif
+	struct servicesmember_struct *next;
+        }servicesmember;
+
+
 /* HOSTSMEMBER structure */
 typedef struct hostsmember_struct{
-	char *host_name;
+	char    *host_name;
+#ifdef NSCORE
+	host    *host_ptr;
+#endif
 	struct hostsmember_struct *next;
         }hostsmember;
 
 
+/* HOSTGROUP structure */
+typedef struct hostgroup_struct{
+	char 	*group_name;
+	char    *alias;
+	hostsmember *members;
+	char    *notes;
+	char    *notes_url;
+	char    *action_url;
+	struct	hostgroup_struct *next;
+	struct	hostgroup_struct *nexthash;
+	}hostgroup;
+
+
 /* HOST structure */
-typedef struct host_struct{
+struct host_struct{
 	char    *name;
+	char    *display_name;
 	char	*alias;
 	char    *address;
         hostsmember *parent_hosts;
+        hostsmember *child_hosts;
+	servicesmember *services;
 	char    *host_check_command;
-	int     check_interval;
+	int     initial_state;
+	double  check_interval;
+	double  retry_interval;
 	int     max_attempts;
 	char    *event_handler;
 	contactgroupsmember *contact_groups;
-	int     notification_interval;
+	contactsmember *contacts;
+	double  notification_interval;
+	double  first_notification_delay;
 	int	notify_on_down;
 	int	notify_on_unreachable;
 	int     notify_on_recovery;
 	int     notify_on_flapping;
+	int     notify_on_downtime;
 	char	*notification_period;
 	char    *check_period;
 	int     flap_detection_enabled;
 	double  low_flap_threshold;
 	double  high_flap_threshold;
+	int     flap_detection_on_up;
+	int     flap_detection_on_down;
+	int     flap_detection_on_unreachable;
 	int     stalk_on_up;
 	int     stalk_on_down;
 	int     stalk_on_unreachable;
@@ -159,6 +309,22 @@ typedef struct host_struct{
 	int     failure_prediction_enabled;
 	char    *failure_prediction_options;
 	int     obsess_over_host;
+	char    *notes;
+	char    *notes_url;
+	char    *action_url;
+	char    *icon_image;
+	char    *icon_image_alt;
+	char    *vrml_image;
+	char    *statusmap_image;
+	int     have_2d_coords;
+	int     x_2d;
+	int     y_2d;
+	int     have_3d_coords;
+	double  x_3d;
+	double  y_3d;
+	double  z_3d;
+	int     should_be_drawn;
+	customvariablesmember *custom_variables;
 #ifdef NSCORE
 	int     problem_has_been_acknowledged;
 	int     acknowledgement_type;
@@ -167,11 +333,17 @@ typedef struct host_struct{
 	int     last_state;
 	int     last_hard_state;
 	char	*plugin_output;
+	char    *long_plugin_output;
 	char    *perf_data;
         int     state_type;
 	int     current_attempt;
+	unsigned long current_event_id;
+	unsigned long last_event_id;
+	unsigned long current_problem_id;
+	unsigned long last_problem_id;
 	double  latency;
 	double  execution_time;
+	int     is_executing;
 	int     check_options;
 	int     notifications_enabled;
 	time_t  last_host_notification;
@@ -190,6 +362,7 @@ typedef struct host_struct{
 	int     notified_on_unreachable;
 	int     current_notification_number;
 	int     no_more_notifications;
+	unsigned long current_notification_id;
 	int     check_flapping_recovery_notification;
 	int     scheduled_downtime_depth;
 	int     pending_flex_downtime;
@@ -204,97 +377,53 @@ typedef struct host_struct{
 	unsigned long modified_attributes;
 	int     circular_path_checked;
 	int     contains_circular_path;
+
+	command *event_handler_ptr;
+	command *check_command_ptr;
+	timeperiod *check_period_ptr;
+	timeperiod *notification_period_ptr;
+	objectlist *hostgroups_ptr;
 #endif
 	struct  host_struct *next;
 	struct  host_struct *nexthash;
-        }host;
-
-
-/* HOSTGROUPMEMBER structure */
-typedef struct hostgroupmember_struct{
-	char    *host_name;
-	struct  hostgroupmember_struct *next;
-        }hostgroupmember;
-
-
-/* HOSTGROUP structure */
-typedef struct hostgroup_struct{
-	char 	*group_name;
-	char    *alias;
-	hostgroupmember *members;
-	struct	hostgroup_struct *next;
-	struct	hostgroup_struct *nexthash;
-	}hostgroup;
-
-
-/* SERVICEGROUPMEMBER structure */
-typedef struct servicegroupmember_struct{
-	char    *host_name;
-	char    *service_description;
-	struct  servicegroupmember_struct *next;
-        }servicegroupmember;
+        };
 
 
 /* SERVICEGROUP structure */
 typedef struct servicegroup_struct{
 	char 	*group_name;
 	char    *alias;
-	servicegroupmember *members;
+	servicesmember *members;
+	char    *notes;
+	char    *notes_url;
+	char    *action_url;
 	struct	servicegroup_struct *next;
 	struct	servicegroup_struct *nexthash;
 	}servicegroup;
 
 
-/* COMMANDSMEMBER structure */
-typedef struct commandsmember_struct{
-	char	*command;
-	struct	commandsmember_struct *next;
-	}commandsmember;
-
-
-/* CONTACT structure */
-typedef struct contact_struct{
-	char	*name;
-	char	*alias;
-	char	*email;
-	char	*pager;
-	char    *address[MAX_CONTACT_ADDRESSES];
-	commandsmember *host_notification_commands;
-	commandsmember *service_notification_commands;	
-	int     notify_on_service_unknown;
-	int     notify_on_service_warning;
-	int     notify_on_service_critical;
-	int     notify_on_service_recovery;
-	int     notify_on_service_flapping;
-	int 	notify_on_host_down;
-	int	notify_on_host_unreachable;
-	int	notify_on_host_recovery;
-	int     notify_on_host_flapping;
-	char	*host_notification_period;
-	char	*service_notification_period;
-	struct	contact_struct *next;
-	struct	contact_struct *nexthash;
-	}contact;
-
-
-
 /* SERVICE structure */
-typedef struct service_struct{
+struct service_struct{
 	char	*host_name;
 	char	*description;
+	char    *display_name;
         char    *service_check_command;
 	char    *event_handler;
-	int	check_interval;
-	int     retry_interval;
+	int     initial_state;
+	double	check_interval;
+	double  retry_interval;
 	int	max_attempts;
 	int     parallelize;
 	contactgroupsmember *contact_groups;
-	int	notification_interval;
+	contactsmember *contacts;
+	double	notification_interval;
+	double  first_notification_delay;
 	int     notify_on_unknown;
 	int	notify_on_warning;
 	int	notify_on_critical;
 	int	notify_on_recovery;
 	int     notify_on_flapping;
+	int     notify_on_downtime;
 	int     stalk_on_ok;
 	int     stalk_on_warning;
 	int     stalk_on_unknown;
@@ -305,6 +434,10 @@ typedef struct service_struct{
 	int     flap_detection_enabled;
 	double  low_flap_threshold;
 	double  high_flap_threshold;
+	int     flap_detection_on_ok;
+	int     flap_detection_on_warning;
+	int     flap_detection_on_unknown;
+	int     flap_detection_on_critical;
 	int     process_performance_data;
 	int     check_freshness;
 	int     freshness_threshold;
@@ -317,24 +450,32 @@ typedef struct service_struct{
 	int     obsess_over_service;
 	int     failure_prediction_enabled;
 	char    *failure_prediction_options;
+	char    *notes;
+	char    *notes_url;
+	char    *action_url;
+	char    *icon_image;
+	char    *icon_image_alt;
+	customvariablesmember *custom_variables;
 #ifdef NSCORE
 	int     problem_has_been_acknowledged;
 	int     acknowledgement_type;
 	int     host_problem_at_last_check;
-#ifdef REMOVED_041403
-	int     no_recovery_notification;
-#endif
 	int     check_type;
 	int	current_state;
 	int	last_state;
 	int	last_hard_state;
 	char	*plugin_output;
+	char    *long_plugin_output;
 	char    *perf_data;
         int     state_type;
 	time_t	next_check;
 	int     should_be_scheduled;
 	time_t	last_check;
 	int	current_attempt;
+	unsigned long current_event_id;
+	unsigned long last_event_id;
+	unsigned long current_problem_id;
+	unsigned long last_problem_id;
 	time_t	last_notification;
 	time_t  next_notification;
 	int     no_more_notifications;
@@ -351,6 +492,7 @@ typedef struct service_struct{
 	int     notified_on_warning;
 	int     notified_on_critical;
 	int     current_notification_number;
+        unsigned long current_notification_id;
 	double  latency;
 	double  execution_time;
 	int     is_executing;
@@ -363,19 +505,19 @@ typedef struct service_struct{
 	unsigned long flapping_comment_id;
 	double  percent_state_change;
 	unsigned long modified_attributes;
+
+	host *host_ptr;
+	command *event_handler_ptr;
+	char *event_handler_args;
+	command *check_command_ptr;
+	char *check_command_args;
+	timeperiod *check_period_ptr;
+	timeperiod *notification_period_ptr;
+	objectlist *servicegroups_ptr;
 #endif
 	struct service_struct *next;
 	struct service_struct *nexthash;
-	}service;
-
-
-/* COMMAND structure */
-typedef struct command_struct{
-	char    *name;
-	char    *command_line;
-	struct command_struct *next;
-	struct command_struct *nexthash;
-        }command;
+	};
 
 
 /* SERVICE ESCALATION structure */
@@ -384,13 +526,18 @@ typedef struct serviceescalation_struct{
 	char    *description;
 	int     first_notification;
 	int     last_notification;
-	int     notification_interval;
+	double  notification_interval;
 	char    *escalation_period;
 	int     escalate_on_recovery;
 	int     escalate_on_warning;
 	int     escalate_on_unknown;
 	int     escalate_on_critical;
 	contactgroupsmember *contact_groups;
+	contactsmember *contacts;
+#ifdef NSCORE
+	service *service_ptr;
+	timeperiod *escalation_period_ptr;
+#endif
 	struct  serviceescalation_struct *next;
 	struct  serviceescalation_struct *nexthash;
         }serviceescalation;
@@ -403,6 +550,7 @@ typedef struct servicedependency_struct{
 	char    *dependent_service_description;
 	char    *host_name;
 	char    *service_description;
+	char    *dependency_period;
 	int     inherits_parent;
 	int     fail_on_ok;
 	int     fail_on_warning;
@@ -412,6 +560,10 @@ typedef struct servicedependency_struct{
 #ifdef NSCORE
 	int     circular_path_checked;
 	int     contains_circular_path;
+
+	service *master_service_ptr;
+	service *dependent_service_ptr;
+	timeperiod *dependency_period_ptr;
 #endif
 	struct servicedependency_struct *next;
 	struct servicedependency_struct *nexthash;
@@ -423,12 +575,17 @@ typedef struct hostescalation_struct{
 	char    *host_name;
 	int     first_notification;
 	int     last_notification;
-	int     notification_interval;
+	double  notification_interval;
 	char    *escalation_period;
 	int     escalate_on_recovery;
 	int     escalate_on_down;
 	int     escalate_on_unreachable;
 	contactgroupsmember *contact_groups;
+	contactsmember *contacts;
+#ifdef NSCORE
+	host    *host_ptr;
+	timeperiod *escalation_period_ptr;
+#endif
 	struct  hostescalation_struct *next;
 	struct  hostescalation_struct *nexthash;
         }hostescalation;
@@ -439,6 +596,7 @@ typedef struct hostdependency_struct{
 	int     dependency_type;
 	char    *dependent_host_name;
 	char    *host_name;
+	char    *dependency_period;
 	int     inherits_parent;
 	int     fail_on_up;
 	int     fail_on_down;
@@ -447,48 +605,16 @@ typedef struct hostdependency_struct{
 #ifdef NSCORE
 	int     circular_path_checked;
 	int     contains_circular_path;
+
+	host    *master_host_ptr;
+	host    *dependent_host_ptr;
+	timeperiod *dependency_period_ptr;
 #endif
 	struct hostdependency_struct *next;
 	struct hostdependency_struct *nexthash;
         }hostdependency;
 
 
-/* EXTENDED HOST INFO structure */
-typedef struct hostextinfo_struct{
-	char *host_name;
-	char *notes;
-	char *notes_url;
-	char *action_url;
-	char *icon_image;
-	char *vrml_image;
-	char *statusmap_image;
-	char *icon_image_alt;
-	int have_2d_coords;
-	int x_2d;
-	int y_2d;
-	int have_3d_coords;
-	double x_3d;
-	double y_3d;
-	double z_3d;
-	int should_be_drawn;
-	struct hostextinfo_struct *next;
-	struct hostextinfo_struct *nexthash;
-        }hostextinfo;
-
-
-/* EXTENDED SERVICE INFO structure */
-typedef struct serviceextinfo_struct{
-	char *host_name;
-	char *description;
-	char *notes;
-	char *notes_url;
-	char *action_url;
-	char *icon_image;
-	char *icon_image_alt;
-	struct serviceextinfo_struct *next;
-	struct serviceextinfo_struct *nexthash;
-        }serviceextinfo;
-
 
 
 /****************** HASH STRUCTURES ********************/
@@ -504,73 +630,77 @@ typedef struct host_cursor_struct{
 
 /********************* FUNCTIONS **********************/
 
-/**** DEBUG functions ****/
-/* RMO: 9/25/01
-   Send debug output to stdout. Does nothing if 'level' is
-   not enabled by a corresponding 'DEBUGn' define.
-   Accepts format string (fmt) and variable-length arg list
-   (as printf does). Prints to stdout and, if NSCGI environment,
-   surrounded with HTML comment delimiters to be viewed through
-   browser's 'view source' option.
-
-   Use as: dbg_print((level,fmt,...)); [NOTE double parens]
- 
-   The macro def below causes dbg_print(()) calls to vaporize
-   if none of the DEBUGn levels are defined.
-*/
-#if defined(DEBUG0) || defined(DEBUG1) || defined(DEBUG2) || defined(DEBUG3) || defined(DEBUG4) || defined(DEBUG5) || defined(DEBUG6) || defined(DEBUG7) || defined(DEBUG8) || defined(DEBUG9) || defined(DEBUG10) || defined(DEBUG11)
-#define dbg_print(args) dbg_print_x args
-#else
-#define dbg_print(args)
-#endif
-
-
 /**** Top-level input functions ****/
-int read_object_config_data(char *,int,int);        /* reads all external configuration data of specific types */
+int read_object_config_data(char *,int,int,int);        /* reads all external configuration data of specific types */
+
 
 /**** Object Creation Functions ****/
-contact *add_contact(char *,char *,char *,char *,char **,char *,char *,int,int,int,int,int,int,int,int,int);	/* adds a contact definition */
+contact *add_contact(char *,char *,char *,char *,char **,char *,char *,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,int);	/* adds a contact definition */
 commandsmember *add_service_notification_command_to_contact(contact *,char *);				/* adds a service notification command to a contact definition */
 commandsmember *add_host_notification_command_to_contact(contact *,char *);				/* adds a host notification command to a contact definition */
-host *add_host(char *,char *,char *,char *,int,int,int,int,int,int,int,char *,int,char *,int,int,char *,int,int,double,double,int,int,int,int,int,char *,int,int,int,int,int);	/* adds a host definition */
+customvariablesmember *add_custom_variable_to_contact(contact *,char *,char *);                         /* adds a custom variable to a service definition */
+host *add_host(char *,char *,char *,char *,char *,int,double,double,int,int,int,int,int,int,double,double,char *,int,char *,int,int,char *,int,int,double,double,int,int,int,int,int,int,int,int,char *,int,int,char *,char *,char *,char *,char *,char *,char *,int,int,int,double,double,double,int,int,int,int,int);	/* adds a host definition */
 hostsmember *add_parent_host_to_host(host *,char *);							/* adds a parent host to a host definition */
+hostsmember *add_child_link_to_host(host *,host *);						        /* adds a child host to a host definition */
 contactgroupsmember *add_contactgroup_to_host(host *,char *);					        /* adds a contactgroup to a host definition */
+contactsmember *add_contact_to_host(host *,char *);                                                     /* adds a contact to a host definition */
+customvariablesmember *add_custom_variable_to_host(host *,char *,char *);                               /* adds a custom variable to a host definition */
 timeperiod *add_timeperiod(char *,char *);								/* adds a timeperiod definition */
+timeperiodexclusion *add_exclusion_to_timeperiod(timeperiod *,char *);                                  /* adds an exclusion to a timeperiod */
 timerange *add_timerange_to_timeperiod(timeperiod *,int,unsigned long,unsigned long);			/* adds a timerange to a timeperiod definition */
-hostgroup *add_hostgroup(char *,char *);								/* adds a hostgroup definition */
-hostgroupmember *add_host_to_hostgroup(hostgroup *, char *);						/* adds a host to a hostgroup definition */
-servicegroup *add_servicegroup(char *,char *);                                                          /* adds a servicegroup definition */
-servicegroupmember *add_service_to_servicegroup(servicegroup *,char *,char *);                          /* adds a service to a servicegroup definition */
+daterange *add_exception_to_timeperiod(timeperiod *,int,int,int,int,int,int,int,int,int,int,int,int);
+timerange *add_timerange_to_daterange(daterange *,unsigned long,unsigned long);
+hostgroup *add_hostgroup(char *,char *,char *,char *,char *);						/* adds a hostgroup definition */
+hostsmember *add_host_to_hostgroup(hostgroup *, char *);						/* adds a host to a hostgroup definition */
+servicegroup *add_servicegroup(char *,char *,char *,char *,char *);                                     /* adds a servicegroup definition */
+servicesmember *add_service_to_servicegroup(servicegroup *,char *,char *);                              /* adds a service to a servicegroup definition */
 contactgroup *add_contactgroup(char *,char *);								/* adds a contactgroup definition */
-contactgroupmember *add_contact_to_contactgroup(contactgroup *,char *);					/* adds a contact to a contact group definition */
+contactsmember *add_contact_to_contactgroup(contactgroup *,char *);					/* adds a contact to a contact group definition */
 command *add_command(char *,char *);									/* adds a command definition */
-service *add_service(char *,char *,char *,int,int,int,int,int,int,char *,int,int,int,int,int,int,int,char *,int,char *,int,int,double,double,int,int,int,int,int,int,char *,int,int,int,int,int);	/* adds a service definition */
+service *add_service(char *,char *,char *,char *,int,int,int,int,double,double,double,double,char *,int,int,int,int,int,int,int,int,char *,int,char *,int,int,double,double,int,int,int,int,int,int,int,int,int,int,char *,int,int,char *,char *,char *,char *,char *,int,int,int);	/* adds a service definition */
 contactgroupsmember *add_contactgroup_to_service(service *,char *);					/* adds a contact group to a service definition */
-serviceescalation *add_serviceescalation(char *,char *,int,int,int,char *,int,int,int,int);             /* adds a service escalation definition */
+contactsmember *add_contact_to_service(service *,char *);                                               /* adds a contact to a host definition */
+serviceescalation *add_serviceescalation(char *,char *,int,int,double,char *,int,int,int,int);          /* adds a service escalation definition */
 contactgroupsmember *add_contactgroup_to_serviceescalation(serviceescalation *,char *);                 /* adds a contact group to a service escalation definition */
-servicedependency *add_service_dependency(char *,char *,char *,char *,int,int,int,int,int,int,int);     /* adds a service dependency definition */
-hostdependency *add_host_dependency(char *,char *,int,int,int,int,int,int);                             /* adds a host dependency definition */
-hostescalation *add_hostescalation(char *,int,int,int,char *,int,int,int);                              /* adds a host escalation definition */
+contactsmember *add_contact_to_serviceescalation(serviceescalation *,char *);                           /* adds a contact to a service escalation definition */
+customvariablesmember *add_custom_variable_to_service(service *,char *,char *);                         /* adds a custom variable to a service definition */
+servicedependency *add_service_dependency(char *,char *,char *,char *,int,int,int,int,int,int,int,char *);     /* adds a service dependency definition */
+hostdependency *add_host_dependency(char *,char *,int,int,int,int,int,int,char *);                             /* adds a host dependency definition */
+hostescalation *add_hostescalation(char *,int,int,double,char *,int,int,int);                           /* adds a host escalation definition */
+contactsmember *add_contact_to_hostescalation(hostescalation *,char *);                                 /* adds a contact to a host escalation definition */
 contactgroupsmember *add_contactgroup_to_hostescalation(hostescalation *,char *);                       /* adds a contact group to a host escalation definition */
-hostextinfo *add_hostextinfo(char *,char *,char *,char *,char *,char *,char *,char *,int,int,double,double,double,int,int); /* adds an extended host info definition */
-serviceextinfo *add_serviceextinfo(char *,char *,char *,char *,char *,char *,char *);                          /* add an extended service info definition */
+
+contactsmember *add_contact_to_object(contactsmember **,char *);                                        /* adds a contact to an object */ 
+customvariablesmember *add_custom_variable_to_object(customvariablesmember **,char *,char *);           /* adds a custom variable to an object */
+
+
+servicesmember *add_service_link_to_host(host *,service *);
+
+
+/*** Object Skiplist Functions ****/
+int init_object_skiplists(void);
+int free_object_skiplists(void);
+int skiplist_compare_text(const char *val1a, const char *val1b, const char *val2a, const char *val2b);
+int skiplist_compare_host(void *a, void *b);
+int skiplist_compare_service(void *a, void *b);
+int skiplist_compare_command(void *a, void *b);
+int skiplist_compare_timeperiod(void *a, void *b);
+int skiplist_compare_contact(void *a, void *b);
+int skiplist_compare_contactgroup(void *a, void *b);
+int skiplist_compare_hostgroup(void *a, void *b);
+int skiplist_compare_servicegroup(void *a, void *b);
+int skiplist_compare_hostescalation(void *a, void *b);
+int skiplist_compare_serviceescalation(void *a, void *b);
+int skiplist_compare_hostdependency(void *a, void *b);
+int skiplist_compare_servicedependency(void *a, void *b);
+
+int get_host_count(void);
+int get_service_count(void);
+
 
 
 /**** Object Hash Functions ****/
-int add_host_to_hashlist(host *);
-int add_service_to_hashlist(service *);
-int add_command_to_hashlist(command *);
-int add_timeperiod_to_hashlist(timeperiod *);
-int add_contact_to_hashlist(contact *);
-int add_contactgroup_to_hashlist(contactgroup *);
-int add_hostgroup_to_hashlist(hostgroup *);
-int add_servicegroup_to_hashlist(servicegroup *);
-int add_hostdependency_to_hashlist(hostdependency *);
 int add_servicedependency_to_hashlist(servicedependency *);
-int add_hostescalation_to_hashlist(hostescalation *);
-int add_serviceescalation_to_hashlist(serviceescalation *);
-int add_hostextinfo_to_hashlist(hostextinfo *);
-int add_serviceextinfo_to_hashlist(serviceextinfo *);
 
 
 /**** Object Search Functions ****/
@@ -580,34 +710,24 @@ hostgroup * find_hostgroup(char *);						                /* finds a hostgroup ob
 servicegroup * find_servicegroup(char *);					                /* finds a servicegroup object */
 contact * find_contact(char *);							                /* finds a contact object */
 contactgroup * find_contactgroup(char *);					                /* finds a contactgroup object */
-contactgroupmember *find_contactgroupmember(char *,contactgroup *);	                        /* finds a contactgroup member object */
 command * find_command(char *);							                /* finds a command object */
 service * find_service(char *,char *);								/* finds a service object */
-hostextinfo *find_hostextinfo(char *);				                                /* find an extended host info object */
-serviceextinfo *find_serviceextinfo(char *,char *);                                             /* find an extended service info object */
 
 
 /**** Object Traversal Functions ****/
-void move_first_service(void);									/* sets up the static memory area for get_next_service */
-service *get_next_service(void);								/* returns the next service, NULL at the end of the list */
-int find_all_services_by_host(char *);							        /* sets up the static memory area for get_next_service_by_host */
-service *get_next_service_by_host(void);							/* returns the next service for the host, NULL at the end of the list */
-void move_first_host(void);									/* sets up the static memory area for get_next_host */
-host *get_next_host(void);									/* returns the next host, NULL at the end of the list */
-void *get_host_cursor(void);					                                /* allocate memory for the host cursor */
-host *get_next_host_cursor(void *v_cursor);							/* return the next host, NULL at the end of the list */
-void free_host_cursor(void *cursor);								/* free allocated cursor memory */
-void *get_next_N(void **hashchain, int hashslots, int *iterator, void *current, void *next);
-
-hostescalation *get_first_hostescalation_by_host(char *);
-hostescalation *get_next_hostescalation_by_host(char *,hostescalation *);
-serviceescalation *get_first_serviceescalation_by_service(char *,char *);
-serviceescalation *get_next_serviceescalation_by_service(char *,char *,serviceescalation *);
-hostdependency *get_first_hostdependency_by_dependent_host(char *);
-hostdependency *get_next_hostdependency_by_dependent_host(char *,hostdependency *);
-servicedependency *get_first_servicedependency_by_dependent_service(char *,char *);
-servicedependency *get_next_servicedependency_by_dependent_service(char *,char *,servicedependency *);
+hostescalation *get_first_hostescalation_by_host(char *, void **);
+hostescalation *get_next_hostescalation_by_host(char *,void **);
+serviceescalation *get_first_serviceescalation_by_service(char *,char *, void **);
+serviceescalation *get_next_serviceescalation_by_service(char *,char *,void **);
+hostdependency *get_first_hostdependency_by_dependent_host(char *, void **);
+hostdependency *get_next_hostdependency_by_dependent_host(char *, void **);
+servicedependency *get_first_servicedependency_by_dependent_service(char *,char *, void **);
+servicedependency *get_next_servicedependency_by_dependent_service(char *,char *,void **);
 
+#ifdef NSCORE
+int add_object_to_objectlist(objectlist **,void *);
+int free_objectlist(objectlist **);
+#endif
 
 
 /**** Object Query Functions ****/
@@ -632,15 +752,17 @@ int number_of_immediate_parent_hosts(host *);				/* counts the number of immedia
 int number_of_total_parent_hosts(host *);				/* counts the number of total parents hosts for a particular host */
 
 #ifdef NSCORE
-int check_for_circular_path(host *,host *);                             /* checks if a circular path exists for a given host */
-int check_for_circular_servicedependency(servicedependency *,servicedependency *,int);   /* checks if a circular dependency exists for a given service */
-int check_for_circular_hostdependency(hostdependency *,hostdependency *,int);   /* checks if a circular dependency exists for a given host */
+int check_for_circular_host_path(host *,host *);                             /* checks if a circular path exists for a given host */
+int check_for_circular_servicedependency_path(servicedependency *,servicedependency *,int);   /* checks if a circular dependency exists for a given service */
+int check_for_circular_hostdependency_path(hostdependency *,hostdependency *,int);   /* checks if a circular dependency exists for a given host */
 #endif
 
 
 /**** Object Cleanup Functions ****/
 int free_object_data(void);                             /* frees all allocated memory for the object definitions */
-int free_extended_data(void);                           /* frees memory allocated to the extended host/service information */
+
+
+
 
 #ifdef __cplusplus
   }
